package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.SportOpeningStatusEnum;
import com.yunzhi.niucai.enums.SportStatusEnum;
import com.yunzhi.niucai.mapper.TaMatchMapper;
import com.yunzhi.niucai.mapper.TaMatchOddsMapper;
import com.yunzhi.niucai.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 比赛  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-02
 */
@Service
public class TaMatchServiceImpl extends ServiceImpl<TaMatchMapper, TaMatch> implements ITaMatchService {

    @Autowired
    ITaMatchWayService iTaMatchWayService;

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaCustomerBettingItemService iTaCustomerBettingItemService;

    @Autowired
    ITaCustomerBettingSportService iTaCustomerBettingSportService;

    @Autowired
    TaMatchMapper taMatchMapper;

    @Autowired
    TaMatchOddsMapper taMatchOddsMapper;

    @Override
    public List<TaMatch> getMixedMatch(String lottery) throws Exception {
        List<TaMatch> taMatches = taMatchMapper.getMixedMatch(lottery, SportStatusEnum.SALE.getCode());
        if (null == taMatches) {
            return null;
        }

        for (TaMatch match : taMatches) {
            List<Map<String, Object>> oddList = taMatchOddsMapper.getByMatch(match.getMatchId());
            if (null != oddList) {
                Map<String, List<Map<String, Object>>> odds = new HashMap<>();

                String key = "";
                List<Map<String, Object>> list = new ArrayList<>();
                for (Map<String, Object> item : oddList) {
                    String wayCode = item.get("wayCode").toString();
                    if (!key.equals(wayCode)) {
                        key = wayCode;
                        list = new ArrayList<>();
                        odds.put(key, list);
                    }
                    list.add(item);
                }

                Map<String, TaMatchWay> groupedByWay = iTaMatchWayService.getGroupedByWay(match.getMatchId(), lottery);

                match.setPlayOdds(odds);
                match.setMatchWay(groupedByWay);
            }
        }

        return taMatches;
    }

    @Override
    public List<TaMatch> getLastResult() throws Exception {
        return taMatchMapper.getLastResult();
    }

    /**
     * 体彩是否中奖
     * @throws Exception
     */
    public void checkAndSetSportPrize() throws Exception {
        List<TaMatch> matchList = taMatchMapper.getLastOpenAndUndoMatch(3);
        if (null == matchList || matchList.size() == 0) {
            return;
        }

        for (TaMatch taMatch : matchList) {
            try {
                taMatch.setOpeningStatus(SportOpeningStatusEnum.DOING.getCode());
                updateById(taMatch);

                String lotteryId = taMatch.getLotteryId();
                switch (lotteryId) {
                    case "football":
                        setFootballPrize(taMatch);
                        break;
                    case "basketball":
                        setBasketballPrize(taMatch);
                        break;
                    default:
                        break;
                }

            } catch (Exception e) {
                // TODO 比如错误日志记录
                taMatch.setOpeningStatus(SportOpeningStatusEnum.READY.getCode());
                updateById(taMatch);
            }
        }
    }

    /**
     * 足球开奖
     * @param taMatch
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    private void setFootballPrize(TaMatch taMatch) throws Exception {
        // 获取当前比赛的投注列表
        List<TaCustomerBettingItem> bettingItemList = iTaCustomerBettingItemService.getUnDealItems(taMatch);
        if (null == bettingItemList || bettingItemList.size() == 0) {
            return;
        }

        for (TaCustomerBettingItem item: bettingItemList) {
            // 主记录
            TaCustomerBetting betting = iTaCustomerBettingService.getById(item.getBettingId());
            // 投注列表
            List<TaCustomerBettingSport> bettingSportList = iTaCustomerBettingSportService.getListByBetting(item.getBettingId(), item.getItemNo());

            for (TaCustomerBettingSport bettingSport : bettingSportList) {

            }
        }
    }

    /**
     * 篮球开奖
     * @param taMatch
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    private void setBasketballPrize(TaMatch taMatch) throws Exception {

    }

    private boolean isWining(TaMatch taMatch, TaCustomerBettingSport bettingSport) {
        return false;
    }
}
